/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import { Component, EventEmitter, Input, OnInit, Output } from "@angular/core";
import { CapabilityRequest } from "../../../../model/capability-request";
import { Capability } from "../../../../model/capability";
import { CapabilityLauncherService } from "../../../../services/capability-launcher.service";
import { WorkflowLauncherService } from "../../../../services/workflow-launcher.service";
import { WorkflowService } from "../../../../services/workflow.service";
import { CapabilityExecution } from "../../../../model/capability-execution";
import { CapabilityRequestService } from "../../../../services/capability-request.service";
import { CapabilityVersion } from "../../../../model/capability-version";
import { CapabilitiesService } from "../../../../services/capabilities.service";

@Component({
  selector: "app-request-operations",
  templateUrl: "./request-operations.component.html",
  styleUrls: ["./request-operations.component.scss"],
})
export class RequestOperationsComponent implements OnInit {
  constructor(
    public capabilityLauncherService: CapabilityLauncherService,
    public workflowLauncherService: WorkflowLauncherService,
    public workflowService: WorkflowService,
    private capabilitiesService: CapabilitiesService,
    private capabilityRequestService: CapabilityRequestService,
  ) {}
  @Input() capabilityRequest: CapabilityRequest;
  @Input() latestVersion: CapabilityVersion;
  @Input() selectedVersion: CapabilityVersion;
  @Input() capability: Capability;
  public capabilityExecution: CapabilityExecution;
  @Output() cartaLaunched: EventEmitter<any> = new EventEmitter<any>();
  public cartaResponse: any;
  public hasBeenClicked: boolean = false;
  public userEmail: string;
  private defaultCC: string;

  // Observer for submitting capability request objects (returns a capability execution)
  public submitRequestObserver = {
    next: (submitResponse: CapabilityExecution) => {
      this.capabilityExecution = submitResponse;
      window.location.reload();
    },
    error: (error) => {
      console.log(error);
    },
  };

  public launchCartaObserver = {
    next: (resp: any) => {
      this.cartaResponse = resp;
      this.hasBeenClicked = true;
      // this.cartaLaunched.emit({ type: "carta", email: this.userEmail });
    },
    error: (error) => {
      console.log(error);

      this.cartaLaunched.emit({ type: "error" });
    },
  };

  public followonRequestObserver = {
    next: (followonResponse: CapabilityRequest) => {
      if (followonResponse.id) {
        // Capability request created; ID found
        // Redirect to request status page
        this.capabilityRequestService.redirectToRequestStatusPage(followonResponse.id);
      }
    },
    error: (error) => {
      console.log(error);
    },
  };

  public cancelVersionObserver = {
    next: () => {
      window.location.reload();
    },
    error: (error) => {
      console.log(error);
    },
  };

  /**
   * Refresh the window which will show that no more versions can be made on this request after it's closed.
   */
  public closeRequestObserver = {
    next: () => {
      window.location.reload();
    },
    error: (error) => {
      console.log(error);
    },
  };

  launchCarta($event) {

    let email = this.userEmail != null ? this.userEmail : "n/a"
    let params = {
      need_data: false,
      notify_ready: true,
      product_locator: null,
      user_email: email,
      parent_wf_request_id: this.selectedVersion.current_execution.current_workflow_request_id,
    };
    this.workflowLauncherService
      .createAndSubmitWorkflow("carta", params)
      .subscribe(this.launchCartaObserver);
  }

  cancelVersion() {
    const requestId = this.selectedVersion.current_execution["current_workflow_request_id"];
    if (requestId == null) {
      console.log("This execution, #" + this.selectedVersion.current_execution.id + ", has no workflow request.");
    } else {
      this.workflowService.abortWorkflow(requestId).subscribe(this.cancelVersionObserver);
    }
  }

  isVersionCancellable(): boolean {
    return (
        this.selectedVersion &&
        this.selectedVersion.state === 'Running' &&
        this.selectedVersion.current_execution.state_name !== 'Stage 2 Review' &&
        this.selectedVersion.current_execution.state_name !== 'Awaiting QA' &&
        this.selectedVersion.current_execution.state_name !== 'Ingesting'
      )
  }

  /**
   * Send a POST request to close (seal) this capability request.
   */
  closeRequest() {
    let request_id = this.capabilityRequest.id;
    this.capabilityRequestService.sendCloseRequest(request_id).subscribe(this.closeRequestObserver);
  }

  /**
   * The request is closable if it's failed or in an error state.
   */
  isRequestCloseable(): boolean {
    return (
      this.selectedVersion &&
      (this.selectedVersion.state === 'Error' || this.selectedVersion.state === 'Failed') &&
      (this.selectedVersion.current_execution.state_name === 'Error' || this.selectedVersion.current_execution.state_name === 'Failed')
      && !this.capabilityRequest.sealed
    );
  }

  public loadDefaultCC() {
    const getAnalystEmailObserver = {
      next: (response) => {
        if (response.resp) {
          if (response.resp.includes("test")){
            this.defaultCC = response.resp
          } else {
            this.defaultCC = "schedsoc@nrao.edu," + response.resp;
          }
        }
      },
      error: (error) => {
        console.error("Failed to load analyst email:", error);
        this.defaultCC = "schedsoc@nrao.edu";
      }
    };

    return this.capabilitiesService.getAnalystEmail().subscribe(getAnalystEmailObserver);
  }

  public getEmailParams() {
    return {
      "destination_email": this.selectedVersion.parameters.user_email,
      "version": this.selectedVersion,
      "workflow_metadata": this.selectedVersion.workflow_metadata
    }
  }

  ngOnInit(): void {
    if (this.selectedVersion.parameters.user_email) {
      this.userEmail = this.selectedVersion.parameters.user_email.toString();
    } else {
      this.userEmail = null;
    }

    this.loadDefaultCC();
  }
}
